@@ -1,17 +1,22 @@
 use strict;
-use warnings;
 
 package Test::Deep::Set;
+use Carp qw( confess );
 
 use Test::Deep::Cmp;
 
+use vars qw( @ISA );
+@ISA = qw( Test::Deep::Cmp );
+
+use Data::Dumper qw(Dumper);
+
 sub init
 {
 	my $self = shift;
 
-	$self->{IgnoreDupes} = shift;
-	$self->{SubSup} = shift;
+	my $dupes = shift;
 
+	$self->{IgnoreDupes} = $dupes;
 	$self->{val} = [];
 
 	$self->add(@_);
@@ -26,12 +31,12 @@ sub descend
 
 	my $IgnoreDupes = $self->{IgnoreDupes};
 
-	my $data = $self->data;
-
-	my $SubSup = $self->{SubSup};
-
 	my $type = $IgnoreDupes ? "Set" : "Bag";
 
+	my %data = (type => $self);
+
+	push(@Test::Deep::Stack, \%data);
+
 	my $diag;
 
 	if (ref $d1 ne 'ARRAY')
@@ -65,32 +70,37 @@ EOM
 			push(@missing, $expect) unless $found;
 		}
 
+		my $got = __PACKAGE__->new($IgnoreDupes, @got);
 
 		my @diags;
-		if (@missing and $SubSup ne "sub")
+		if (@missing)
 		{
 			push(@diags, "Missing: ".nice_list(\@missing));
 		}
 
-		if (@got and $SubSup ne "sup")
+		if (@got)
 		{
-			my $got = __PACKAGE__->new($IgnoreDupes, "", @got);
 			push(@diags, "Extra: ".nice_list($got->{val}));
 		}
 
 		$diag = join("\n", @diags);
 	}
 
+	my $ok;
 	if ($diag)
 	{
-		$data->{diag} = $diag;
+		$ok = 0;
 
-		return 0;
+		$data{diag} = $diag;
 	}
 	else
 	{
-		return 1;
+		$ok = 1;
 	}
+
+	pop @Test::Deep::Stack if $ok;
+
+	return $ok;
 }
 
 sub diagnostics
@@ -99,8 +109,6 @@ sub diagnostics
 	my ($where, $last) = @_;
 
 	my $type = $self->{IgnoreDupes} ? "Set" : "Bag";
-	$type = "Sub$type" if $self->{SubSup} eq "sub";
-	$type = "Super$type" if $self->{SubSup} eq "sup";
 
 	my $error = $last->{diag};
 	my $diag = <<EOM;
@@ -148,7 +156,7 @@ sub add
 
 	# so we can compare 2 Test::Deep::Set objects using array comparison
 
-	@$already = sort {(defined $a ? $a : "") cmp (defined $b ? $b : "")} @$already;
+	@$already = sort @$already;
 }
 
 sub nice_list
@@ -164,11 +172,19 @@ sub nice_list
 	# sort them so we can predict the diagnostic output
 
 	return join(", ",
-		(map {Test::Deep::render_val($_)} sort {(defined $a ? $a : "") cmp (defined $b ? $b : "")} @scalars),
+		(map {"'$_'"} sort @scalars),
 		@ref_string
 	);
 }
 
+sub render_stack
+{
+	my $self = shift;
+	my $var = shift;
+
+	return $var;
+}
+
 sub compare
 {
 	my $self = shift;
